package application;

import java.io.*;
import java.util.List;
import java.util.LinkedList;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import project.ProjectManager;
import language.java.*;
import static logging.Logging.logger;

public class Actions {

    public static class NewProjectAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            JFileChooser chooser = new JFileChooser();

            if (chooser.showSaveDialog(Application.getFrame()) == JFileChooser.APPROVE_OPTION) {
                String name = chooser.getSelectedFile().getName();
                String folder = chooser.getSelectedFile().getParentFile().getAbsolutePath() + "/" + name;

                File parent = new File(folder);
                if (parent.exists() && !parent.isDirectory()) {
                    JOptionPane.showMessageDialog(Application.getFrame(), "In directorul curent nu se poate crea proiect cu numele dorit.");
                    return;
                }

                ProjectManager.openProject(folder + "/" + name + ".proj", name);
            }
        }

    }

    public static class NewFileAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            JFileChooser chooser = new JFileChooser();
            if (chooser.showSaveDialog(Application.getFrame()) == JFileChooser.APPROVE_OPTION) {
                    ProjectManager.openFile(chooser.getSelectedFile().getAbsolutePath());
            }
        }

    }

    public static class OpenProjectAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            JFileChooser chooser = new JFileChooser();
            FileFilter filter = new FileFilter() {
                public boolean accept(File f) {
                    return f.getName().toLowerCase().endsWith(".proj") || f.isDirectory();
                }
                public String getDescription() {
                    return "Project Files";
                }
            };
            chooser.setFileFilter(filter);

            if (chooser.showOpenDialog(Application.getFrame()) == JFileChooser.APPROVE_OPTION) {
                ProjectManager.openProject(chooser.getSelectedFile().getAbsolutePath());
            }
        }

    }

    public static class OpenFileAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            JFileChooser chooser = new JFileChooser();
            if (chooser.showOpenDialog(Application.getFrame()) == JFileChooser.APPROVE_OPTION) {
                    ProjectManager.openFile(chooser.getSelectedFile().getAbsolutePath());
            }
        }

    }

    public static class SaveAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.save();
        }

    }

    public static class SaveAllAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Editor editor = Application.getEditor();
            for (int i = 0; i < editor.getTabCount(); ++i) {
                ((EditorContent) editor.getComponentAt(i)).getEditorPane().save();
            }
        }

    }

    public static class CloseAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            if (editorPane.close())
                Application.getEditor().remove(component);
        }

    }

    public static class CloseAllAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Editor editor = Application.getEditor();
            for (int i = 0; i < editor.getTabCount(); ++i) {
                ((EditorContent) editor.getComponentAt(i)).getEditorPane().close();
            }
            editor.removeAll();
        }

    }

    public static class ExitAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }

    }

    public static class UndoAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.undo();
        }

    }

    public static class RedoAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.redo();
        }

    }

    public static class CutAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.cut();
        }

    }

    public static class CopyAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.copy();
        }

    }

    public static class PasteAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.paste();
        }

    }

    public static class SelectAllAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.requestFocus();
            editorPane.selectAll();
        }

    }

    public static class GoToAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.goToLine();
        }

    }

    public static class CompileAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.compile();
        }

    }

    public static class RunAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            editorPane.run();
        }

    }

    public static class RunArgAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            String args = JOptionPane.showInputDialog("Arguments:");
            if (args != null)
                editorPane.run(args);
        }

    }

    public static class StopAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            language.java.JavaRunner.kill();
        }

    }

    public static class PreferencesAction implements ActionListener {

        public void actionPerformed(ActionEvent arg0) {
            new PreferencesDialog();
        }

    }

    public static class PreferencesDialog extends JDialog implements ActionListener {
        private JTextField tabSize = new JTextField(new Integer(Config.TAB_SIZE).toString());
        private JComboBox javaColors;
        private JCheckBox autosave = new JCheckBox("Autosave", Config.AUTOSAVE);

        public PreferencesDialog() {
            super(Application.getFrame(), "Preferences");

            String[] javaColorOptions = { "JavaDefaultColorScheme", "JavaShineColorScheme" };
            javaColors = new JComboBox(javaColorOptions);
            javaColors.setSelectedItem(Config.javaColorScheme);

            JPanel labels = new JPanel();
            labels.setLayout(new GridLayout(2, 1));
            labels.add(new JLabel("Tab size:"));
            labels.add(new JLabel("Java Color Scheme: "));

            JPanel options = new JPanel();
            options.setLayout(new GridLayout(2, 1));
            options.add(tabSize);
            options.add(javaColors);

            JPanel topPanel = new JPanel();
            topPanel.setLayout(new FlowLayout());
            topPanel.add(labels);
            topPanel.add(options);

            JPanel buttonsPanel = new JPanel();
            JButton ok = new JButton("Ok");
            ok.addActionListener(this);
            JButton cancel = new JButton("Cancel");
            cancel.addActionListener(this);
            buttonsPanel.add(ok);
            buttonsPanel.add(cancel);

            setLayout(new BorderLayout());
            add(topPanel, BorderLayout.NORTH);
            add(autosave, BorderLayout.CENTER);
            add(buttonsPanel, BorderLayout.SOUTH);

            setLocationRelativeTo(null);
            setVisible(true);
            pack();
        }

        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("Ok"))
                try {
                    int value = Integer.parseInt(tabSize.getText());
                    Config.TAB_SIZE = value;
                    Config.javaColorScheme = (String) javaColors.getSelectedItem();
                    Config.AUTOSAVE = autosave.isSelected();
                    EditorContent editorContent = (EditorContent) Application.getEditor().getSelectedComponent();
                    if (editorContent != null) {
                        EditorPane editorPane = editorContent.getEditorPane();
                        if (editorPane.getEditorKit() instanceof JavaEditorKit) {
                            ((JavaEditorKit) editorPane.getEditorKit()).getStylePreferences().updateColorScheme();
                            editorPane.repaint();
                        }
                    }
                    Config.save();
                    dispose();
                } catch (NumberFormatException ex) {
                    return;
                }
            dispose();
        }
    }

    public static class SearchInFileAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();
            String data = editorPane.getText();

            int startPos = editorPane.getCaretPosition();
            String lastString = "";
            while(true) {
                String str = JOptionPane.showInputDialog("Search string in file:", lastString);
                if (str == null || str.equals(""))
                    break;
                int index = data.indexOf(str, startPos);
                if (index == -1)
                    if (startPos == 0) {
                        JOptionPane.showMessageDialog(Application.getFrame(),
                                "No such string in this file!");
                        break;
                    } else {
                        startPos = 0;
                        index = data.indexOf(str, startPos);
                        if (index == -1) {
                            JOptionPane.showMessageDialog(Application.getFrame(),
                                    "No such string in this file!");
                            break;
                        }
                    }
                editorPane.setSelectionStart(index);
                editorPane.setSelectionEnd(index + str.length());
                startPos = editorPane.getSelectionEnd();
                lastString = str;
            }
        }

    }

    public static class FileReopenAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            ProjectManager.openFile(e.getActionCommand());
        }

    }

    public static class SearchInProjectAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {

            Component component = Application.getEditor().getSelectedComponent();
            if (component == null)
                return;
            EditorPane editorPane = ((EditorContent) component).getEditorPane();

            String str = JOptionPane.showInputDialog("Search in project");
            if (str == null)
                return;

            List <Result> results = new LinkedList <Result> ();

            for (String path : ProjectManager.getAllFilePaths(editorPane.getPath())) {

                File f = new File(path);

                char[] buffer = new char[(int)f.length()];
                try {
                    new FileReader(f).read(buffer);
                } catch (Exception exp) {
                    logger.throwing("application.Actions$SearchInProjectAction",
                            "actionPerformed", exp);
                }

                int startPos = 0, index, lineNumber = 0, lastLineNumber = -1;
                String data = new String(buffer);
                while (true) {
                    index = data.indexOf(str, startPos);
                    if (index == -1)
                        break;

                    for (int i = startPos; i < index; ++i)
                        if (buffer[i] == '\n')
                            ++lineNumber;
                    int start = index;
                    while (start >= 0 && buffer[start] != '\n')
                        --start;
                    ++start;
                    int end = index;
                    while (end < buffer.length && buffer[end] != '\n')
                        ++end;

                    if (lineNumber != lastLineNumber)
                        results.add(new Result(index, lineNumber + 1,
                                data.substring(start, end), path));
                    lastLineNumber = lineNumber;
                    startPos = index + str.length();
                }

            }

            Application.getConsole().setResults(results);

        }

    }

    public static class Result {

        int position, lineNumber;
        String lineData, fileName;

        public Result(int position, int lineNumber, String lineData, String fileName) {
            this.position = position;
            this.lineNumber = lineNumber;
            this.lineData = lineData;
            this.fileName = fileName;
        }

        public String toString() {
            return "Line " + lineNumber + " (" + fileName + "): " + lineData;
        }

    }

}
