package application;

import java.util.*;
import java.awt.*;
import javax.swing.*;

import plugins.*;
import logging.Logging;
import project.File;
import project.ProjectManager;

public class Application {

    static AppFrame frame = new AppFrame();
    static Editor editor = new Editor();
    static ProjTree projectsTree = new ProjTree();
    static Console console = new Console();
    static FunctionsTree functionsTree = new FunctionsTree();
    static ToolBar toolbar = new ToolBar();
    static MenuBar menuBar = new MenuBar();

    // Put together all parts of the application and let's go.
    public static void run() {

        JEditorPane.registerEditorKitForContentType("text/java",
                "language.java.JavaEditorKit");
        JEditorPane.registerEditorKitForContentType("text/xml",
                "language.xml.XMLEditorKit");
        JEditorPane.registerEditorKitForContentType("text/html",
                "language.html.HTMLEditorKit");

        // Configure stuff.
        Config.init();

        // Load plugins.
        PluginManager.init();

        // Put the components on the frame.
        JSplitPane leftSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                new JScrollPane(projectsTree), editor);
        leftSplit.setResizeWeight(0.33);
        leftSplit.setContinuousLayout(true);

        JSplitPane rightSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                leftSplit, functionsTree);
        rightSplit.setResizeWeight(0.75);
        rightSplit.setContinuousLayout(true);

        JSplitPane verticalSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                rightSplit, console);
        verticalSplit.setResizeWeight(0.75);
        verticalSplit.setContinuousLayout(true);

        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.add("Center", verticalSplit);

        Container content = frame.getContentPane();
        content.setLayout(new BorderLayout());
        content.add("North", toolbar);
        content.add("Center", panel);

        // Set up the menu bar.
        frame.setJMenuBar(menuBar);

        // Display the application window.
        frame.setVisible(true);

        if (Config.AUTOSAVE)
            new AutoSaveThread();
    }

    public static Observer addEditorTab(String tabName) {
        return editor.addTab(tabName);
    }

    public static Observer getProjectsTree() {
        return projectsTree;
    }

    public static Editor getEditor() {
        return editor;
    }

    public static MenuBar getMenuBar() {
        return menuBar;
    }

    public static Console getConsole() {
        return console;
    }

    public static FunctionsTree getFunctionTree() {
        return functionsTree;
    }

    public static ToolBar getToolBar() {
        return toolbar;
    }

    public static void updatePluginsMenu() {
        menuBar.updatePluginsMenu();
    }

    public static AppFrame getFrame() {
        return frame;
    }

    private static class AutoSaveThread extends Thread {
        public AutoSaveThread() {
            start();
        }

        public void run() {
            Editor editor = Application.getEditor();

            while (true) {
                int index = 0;
                while (index < editor.getTabCount()) {
                    try {
                        EditorContent editorContent = (EditorContent) editor.getComponentAt(index);
                        if (editorContent != null) {
                            EditorPane editorPane = editorContent.getEditorPane();
                            String path = editorPane.getPath();
                            ProjectManager.saveFile(path + ".tmp", editorPane.getText());
                            File.renameFile(path + ".tmp", path);
                            editorPane.setSaved();
                        }
                    } catch (Exception e) {
                        Logging.logger.throwing("AutoSaveThread", "run", e);
                    }
                    index++;
                }
                try {
                    Thread.sleep(60000);
                } catch (InterruptedException e) {
                }
            }
        }
    }

}
