package application;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import language.*;

public class CCPopup {

    static boolean visible = false;
    static int x = 0, y = 0;
    static Popup popup = null;
    static LinkedList <CCInfo> info = null;
    static LinkedList <Character> filter = null;
    static JList suggestionsList = null;

    public static synchronized void show() {
        visible = true;
    }

    public static synchronized void hide() {
        visible = false;
        if (popup != null) {
            popup.hide();
            popup = null;
        }
        info = null;
        filter = null;
        suggestionsList = null;
    }

    public static synchronized boolean isVisible() {
        return visible && popup != null;
    }

    private static void displayPopup() {

        if (!visible || info == null || filter == null)
            return;

        if (popup != null) {
            popup.hide();
            popup = null;
        }

        LinkedList <CCInfo> data = filterData();

        suggestionsList = new JList();
        suggestionsList.setFocusable(false);
        suggestionsList.setBackground(new Color(0.95f, 0.95f, 0.3f));
        suggestionsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        suggestionsList.setListData(data.toArray());
        suggestionsList.addMouseListener(new MouseAction());

        JScrollPane scroll = new JScrollPane(suggestionsList);
        scroll.setFocusable(false);
        scroll.setPreferredSize(new Dimension(350, 150));

        popup = PopupFactory.getSharedInstance().
            getPopup(Application.getEditor(), scroll, x, y);
        if (data.size() > 0) {
            suggestionsList.setSelectedIndex(0);
            popup.show();
        } else {
            popup = null;
        }

    }

    public static synchronized void setList(LinkedList <CCInfo> i) {
        if (!visible)
            return;
        info = i;
        displayPopup();
    }

    public static synchronized void setFilter(LinkedList <Character> f) {
        if (!visible)
            return;
        filter = f;
        displayPopup();
    }

    public static synchronized void setPosition(int xx, int yy) {
        x = xx;
        y = yy;
        if (popup != null) {
            popup.hide();

            JScrollPane scroll = new JScrollPane(suggestionsList);
            scroll.setFocusable(false);
            scroll.setPreferredSize(new Dimension(250, 150));

            popup = PopupFactory.getSharedInstance().
                getPopup(Application.getEditor(), scroll, x, y);
            popup.show();
        }
    }

    public static synchronized void selectNext() {
        if (!visible || suggestionsList == null)
            return;

        int newIndex = (suggestionsList.getSelectedIndex() + 1)
            % suggestionsList.getModel().getSize();

        suggestionsList.setSelectedIndex(newIndex);
        suggestionsList.ensureIndexIsVisible(newIndex);
    }

    public static synchronized void selectPrev() {
        if (!visible || suggestionsList == null)
            return;

        int index = suggestionsList.getSelectedIndex();
        int size = suggestionsList.getModel().getSize();

        if (index == -1) {
            suggestionsList.setSelectedIndex(size - 1);
            suggestionsList.ensureIndexIsVisible(size - 1);
        } else {
            suggestionsList.setSelectedIndex((index + size - 1) % size);
            suggestionsList.ensureIndexIsVisible((index + size - 1) % size);
        }
    }

    public static LinkedList <CCInfo> filterData() {

        LinkedList <CCInfo> result = new LinkedList <CCInfo> ();

        for (CCInfo i : info) {

            int p = 0;
            String str = i.getInfo();
            boolean ok = true;

            for (Character ch : filter) {
                if (p >= str.length() || ch != str.charAt(p)) {
                    ok = false;
                    break;
                }
                ++p;
            }

            if (ok)
                result.add(i);

        }

        return result;

    }

    public static String getSubmitData() {

        if (!visible || popup == null || suggestionsList == null || filter == null)
            return "";

        int index = suggestionsList.getSelectedIndex();
        if (index == -1)
            return "";

        CCInfo selected = (CCInfo) suggestionsList.getModel().getElementAt(index);
        String data = selected.getInfo();

        int brIndex = data.indexOf("(");
        if (brIndex == -1)
            return data.substring(filter.size());
        else
            if (filter.size() <= brIndex + 1)
                return data.substring(filter.size(), brIndex + 1);
            else
                return "";

    }

    static class MouseAction implements MouseListener {

        public void mouseClicked(MouseEvent e) {
            EditorPane editor = ((EditorContent) Application.getEditor().getSelectedComponent())
                .getEditorPane();
            editor.submitData();
            CCPopup.hide();
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mousePressed(MouseEvent e) {
        }

        public void mouseReleased(MouseEvent e) {
        }

    }

}
