package application;

import java.io.*;
import java.util.*;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import application.Actions.FileReopenAction;

import static logging.Logging.logger;

public class Config {
    public static int TAB_SIZE = 4;
    public static boolean AUTOSAVE = false;
    public static String javaColorScheme = "default";
    public static ArrayDeque<String> lastFiles = new ArrayDeque<String>();

    private static HashMap <String, Boolean> activatedPlugins =
        new HashMap <String, Boolean> ();

    public static void init() {
            BufferedReader in;

            try {
                in = new BufferedReader(new FileReader("config.ini"));
            } catch (FileNotFoundException e) {
                logger.throwing("Config", "init", e);
                try {
                    (new File("config.ini")).createNewFile();
                    save();
                } catch (IOException e1) {
                    logger.throwing("Config", "init", e);
                }
                return;
            }

            try {
                try {
                    TAB_SIZE = Integer.parseInt(in.readLine());
                } catch (NumberFormatException e) {
                    TAB_SIZE = 4;
                }

                AUTOSAVE = new Boolean(in.readLine());

                javaColorScheme = in.readLine();

                lastFiles = new ArrayDeque<String>();
                for (int i = 0; i < 5; i++)
                    lastFiles.add(in.readLine());

                activatedPlugins = new HashMap <String, Boolean> ();
                while (in.ready())
                    activatedPlugins.put(in.readLine(), false);

                updateMenu();
            } catch (IOException e) {
                logger.throwing("Config", "init", e);
            }

    }

    public static void updateMenu() {
        JMenu menuOpen = Application.getMenuBar().getMenuOpen();
        menuOpen.removeAll();
        Iterator<String> it = Config.lastFiles.descendingIterator();
        while (it.hasNext()) {
            JMenuItem item = new JMenuItem(it.next());
            item.addActionListener(new FileReopenAction());
            menuOpen.add(item);
        }
    }

    public static void save() {
        try {
            FileWriter out = new FileWriter("config.ini");
            out.write(TAB_SIZE + "\n");
            out.write(AUTOSAVE + "\n");
            out.write(javaColorScheme + "\n");

            Iterator<String> it = lastFiles.iterator();
            int i = 0;
            while (it.hasNext()) {
                out.write(it.next() + "\n");
                i++;
            }

            while (i++ < 5)
                out.write("\n");

            for (Map.Entry<String, Boolean> entry : activatedPlugins.entrySet())
                if (!entry.getValue())
                    out.write(entry.getKey() + "\n");
            out.close();
        } catch (IOException e) {
            logger.throwing("Config", "save", e);
        }
    }

    public static void addFile(String file) {
        Iterator<String> it = lastFiles.iterator();
        while (it.hasNext())
            if (it.next().equals(file))
                return;

        if (lastFiles.size() == 5)
            lastFiles.removeFirst();
        lastFiles.add(file);
        save();
        updateMenu();
    }

    public static boolean isActivePlugin(String name) {
        Boolean b = activatedPlugins.get(name);
        return b == null || b.equals(true);
    }

    public static void setActivatedPlugin(String name, boolean isActivated) {
        activatedPlugins.put(name, isActivated);
        save();
    }

}
