package application;

import java.io.*;
import java.util.*;
import java.awt.event.*;
import java.awt.Color;

import javax.swing.*;
import javax.swing.text.*;

import language.java.JavaRunner;
import project.ProjectManager;
import static logging.Logging.logger;

public class Console extends JTabbedPane {
    JEditorPane errorConsole = new JEditorPane();
    JEditorPane runnerConsole = new JEditorPane();
    JEditorPane seoConsole = new JEditorPane(); 
    JList searchResults = new JList();

    public Console() {
        errorConsole.setEditable(false);
        errorConsole.setBackground(Color.white);
        runnerConsole.setEditable(false);
        runnerConsole.setBackground(Color.white);
        seoConsole.setEditable(false);
        seoConsole.setBackground(Color.decode("#CCF2FF"));
      
        searchResults.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        add(new JScrollPane(errorConsole), "Compiler console");
        add(new JScrollPane(runnerConsole), "Runner console");
        add(new JScrollPane(searchResults), "Search results");
        add(new JScrollPane(seoConsole), "SEO console");
    }
       	  

    public void processStarted(InputStream in, OutputStream out) {
        runnerConsole.setText(null);
        runnerConsole.setEditable(true);
        runnerConsole.setDocument(new TerminalDocument(in, out));
    }

    public void processEnded() {
        runnerConsole.setEditable(false);
    }

    public JEditorPane getErrorConsole() {
        return errorConsole;
    }

    public JEditorPane getRunnerConsole() {
        return runnerConsole;
    }

    public class TerminalDocument extends PlainDocument {
        private static final int ATTRIBUTE = 0;
        private InputStream in = null;
        private OutputStream out = null;
        private int editStartPos = 0;

        public TerminalDocument(InputStream in, OutputStream out) {
            this.in = in;
            this.out = out;
            setDocumentFilter(new TerminalFilter());
            new Reader();
        }

        protected void insertUpdate(DefaultDocumentEvent chng, AttributeSet attr) {
            super.insertUpdate(chng, attr);

            String buffer = null;
            int index;

            if (attr != null && attr.isDefined(ATTRIBUTE)) {
                try {
                    buffer = getText(editStartPos, chng.getOffset() - editStartPos);
                } catch (BadLocationException e) {
                    logger.throwing("TerminalDocument", "insertUpdate", e);
                }
                try {
                    out.write(buffer.getBytes());
                    out.flush();
                } catch (IOException e) {
                    logger.throwing("TerminalDocument", "insertUpdate", e);
                }
                editStartPos = chng.getOffset() + chng.getLength();
                return;
            }

            try {
                buffer = getText(editStartPos, getLength() - editStartPos);
            } catch (BadLocationException e) {
                logger.throwing("TerminalDocument", "insertUpdate", e);
            }

            if ((index = buffer.lastIndexOf('\n')) != -1) {
                try {
                    out.write(buffer.substring(0, index + 1).getBytes());
                    out.flush();
                } catch (IOException e) {
                    logger.throwing("TerminalDocument", "insertUpdate", e);
                }
                editStartPos = editStartPos + index + 1;
            }
        }

        public class TerminalFilter extends DocumentFilter {

            public void insertString(DocumentFilter.FilterBypass fb, int offset,
                                                String string, AttributeSet attr) throws BadLocationException {

                if ((attr != null && attr.isDefined(ATTRIBUTE)) || offset >= editStartPos)
                    super.insertString(fb, offset, string, attr);
            }

            public void replace(DocumentFilter.FilterBypass fb, int offset, int length,
                                                    String string, AttributeSet attr) throws BadLocationException {

                if ((attr != null && attr.isDefined(ATTRIBUTE)) || offset >= editStartPos)
                    super.insertString(fb, offset, string, attr);
            }

            public void remove(DocumentFilter.FilterBypass fb, int offset, int length)
                                                                              throws BadLocationException {
                if (offset >= editStartPos)
                    super.remove(fb, offset, length);
            }

        }

        public class Reader extends Thread {
            Reader() {
                start();
            }

            public void run() {
                byte[] buffer = new byte[1024];
                int bytesRead = -1;

                try {
                    while ((bytesRead = in.read(buffer)) != -1) {
                        SimpleAttributeSet attrs = new SimpleAttributeSet();
                        attrs.addAttribute(ATTRIBUTE, 0);

                        try {
                            insertString(getLength(), new String(buffer, 0, bytesRead), attrs);
                            runnerConsole.setCaretPosition(getLength());
                        } catch (BadLocationException e) {
                            logger.throwing("Console", "Reader", e);
                        }
                    }
                } catch (IOException e) {
                    logger.throwing("Console", "Reader", e);
                    return;
                }

                in = null;
                out = null;
                runnerConsole.setEditable(false);
                JavaRunner.ended();
            }
        }
    }

    public void setResults(List <Actions.Result> list) {
        searchResults.setListData(list.toArray());
        searchResults.addMouseListener(new SearchResultMouseAction());
        setSelectedIndex(2);
    }

    class SearchResultMouseAction implements MouseListener {

        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() == 2) {
                int index = searchResults.getSelectedIndex();
                if (index != -1) {
                    Actions.Result result = (Actions.Result) searchResults.
                        getModel().getElementAt(index);
                    EditorPane editor = ProjectManager.getFileEditor(result.fileName);
                    editor.setCaretPosition(result.position);
                    editor.requestFocus();
                }
            }
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mousePressed(MouseEvent e) {
        }

        public void mouseReleased(MouseEvent e) {
        }

    }

}

