package application;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.text.*;

import language.*;

public class FunctionsTree extends JPanel {

    JTabbedPane tabbedPane;
    JScrollPane scrollPane;
    JTree tree;
    TreeUpdater updater;

    public FunctionsTree() {
        setLayout(new BorderLayout());

        tabbedPane = new JTabbedPane();
        tree = new JTree();
        scrollPane = new JScrollPane(tree);

        tabbedPane.addTab("Functions tree", scrollPane);
        add(tabbedPane);

        tree.setToggleClickCount(Integer.MAX_VALUE);
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.addFocusListener(new TreeFocusListener());
        tree.addMouseListener(new MouseAction());

        updater = new TreeUpdater();
        updater.start();
    }

    public synchronized void update() {

        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                DefaultMutableTreeNode root = new DefaultMutableTreeNode();

                Component editorTab = Application.getEditor().getSelectedComponent();
                if (editorTab != null) {
                    Document doc = ((EditorContent) editorTab).getEditorPane().getDocument();
                    if (doc instanceof XDocument) {
                        XDocument.FunctionsTree func = ((XDocument) doc).getFunctionsTree();
                        if (func != null)
                            buildTree(root, func);
                    }
                }

                ((DefaultTreeModel) tree.getModel()).setRoot(root);
                tree.setRootVisible(false);

                for (int i = 0; i < tree.getRowCount(); i++)
                    tree.expandRow(i);
            }

        });

    }

    void buildTree(DefaultMutableTreeNode root, XDocument.FunctionsTree func) {
        for (XDocument.FunctionsTree tree : func.list) {
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(tree);
            root.add(node);
            buildTree(node, tree);
        }
    }

    class TreeUpdater extends Thread {

        public void run() {
            while (true) {
                update();
                try {
                    sleep(3000);
                } catch (InterruptedException e) {
                    return;
                }
            }
        }

    }

    class TreeFocusListener implements FocusListener {

        public void focusGained(FocusEvent e) {
            updater.interrupt();
        }

        public void focusLost(FocusEvent e) {
            updater = new TreeUpdater();
            updater.start();
        }

    }

    class MouseAction extends MouseAdapter {

        public void mousePressed(MouseEvent e) {
            if (e.getClickCount() == 2) {
                TreePath path = tree.getPathForLocation(e.getX(), e.getY());
                if (path == null)
                    return;
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
                XDocument.FunctionsTree func = (XDocument.FunctionsTree) node.getUserObject();

                Component editorTab = Application.getEditor().getSelectedComponent();
                if (editorTab == null)
                    return;
                EditorPane editorPane = ((EditorContent) editorTab).getEditorPane();
                editorPane.setCaretPosition(func.pos);
            }
        }

    }

}
