package application;

import java.awt.event.*;

import javax.swing.*;
import plugins.*;
import static application.Actions.*;

public class MenuBar extends JMenuBar {

    JMenu menuFile, menuOpen, menuEdit, menuSearch, menuProject, menuPlugins;

    public MenuBar() {
        super();

        JMenuItem item = null;
        JMenu submenu = null;

        // >>> File <<<
        menuFile = new JMenu("File");
        menuFile.setMnemonic(KeyEvent.VK_F);

        submenu = new JMenu("New...");
        item = new JMenuItem("Project");
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.SHIFT_DOWN_MASK | InputEvent.CTRL_DOWN_MASK));
        item.addActionListener(new NewProjectAction());
        submenu.add(item);
        item = new JMenuItem("File");
        item.addActionListener(new NewFileAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_DOWN_MASK));
        submenu.add(item);
        menuFile.add(submenu);
        menuFile.addSeparator();

        item = new JMenuItem("Open Project");
        item.addActionListener(new OpenProjectAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.SHIFT_DOWN_MASK | InputEvent.CTRL_DOWN_MASK));
        menuFile.add(item);
        item = new JMenuItem("Open File");
        item.addActionListener(new OpenFileAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK));
        menuFile.add(item);
        menuOpen = new JMenu("Open ...");
        menuFile.add(menuOpen);
        menuFile.addSeparator();

        item = new JMenuItem("Save");
        item.addActionListener(new SaveAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK));
        menuFile.add(item);
        item = new JMenuItem("Save all");
        item.addActionListener(new SaveAllAction());
        menuFile.add(item);
        menuFile.addSeparator();

        item = new JMenuItem("Close");
        item.addActionListener(new CloseAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK));
        menuFile.add(item);
        item = new JMenuItem("Close all");
        item.addActionListener(new CloseAllAction());
        menuFile.add(item);
        menuFile.addSeparator();

        item = new JMenuItem("Exit");
        item.addActionListener(new ExitAction());
        menuFile.add(item);

        // >>> Edit <<<
        menuEdit = new JMenu("Edit");
        menuEdit.setMnemonic(KeyEvent.VK_E);

        item = new JMenuItem("Undo");
        item.addActionListener(new UndoAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_DOWN_MASK));
        menuEdit.add(item);
        item = new JMenuItem("Redo");
        item.addActionListener(new RedoAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_DOWN_MASK));
        menuEdit.add(item);
        menuEdit.addSeparator();

        item = new JMenuItem("Cut");
        item.addActionListener(new CutAction());
        menuEdit.add(item);
        item = new JMenuItem("Copy");
        item.addActionListener(new CopyAction());
        menuEdit.add(item);
        item = new JMenuItem("Paste");
        item.addActionListener(new PasteAction());
        menuEdit.add(item);
        menuEdit.addSeparator();

        item = new JMenuItem("Select all");
        item.addActionListener(new SelectAllAction());
        menuEdit.add(item);
        item = new JMenuItem("Preferences");
        item.addActionListener(new PreferencesAction());
        menuEdit.add(item);

        // >>> Search <<<
        menuSearch = new JMenu("Search");
        menuSearch.setMnemonic(KeyEvent.VK_S);

        item = new JMenuItem("...in file");
        item.addActionListener(new SearchInFileAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_DOWN_MASK));
        menuSearch.add(item);
        item = new JMenuItem("...in project");
        item.addActionListener(new SearchInProjectAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_DOWN_MASK));
        menuSearch.add(item);
        item = new JMenuItem("Go to line");
        item.addActionListener(new GoToAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_DOWN_MASK));
        menuSearch.add(item);

        // >>> Project <<<
        menuProject = new JMenu("Project");
        menuProject.setMnemonic(KeyEvent.VK_R);

        item = new JMenuItem("Compile");
        item.addActionListener(new CompileAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F9, InputEvent.CTRL_DOWN_MASK));
        menuProject.add(item);
        item = new JMenuItem("Run");
        item.addActionListener(new RunAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F11, InputEvent.CTRL_DOWN_MASK));
        menuProject.add(item);
        item = new JMenuItem("Run with arguments");
        item.addActionListener(new RunArgAction());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F10, InputEvent.CTRL_DOWN_MASK));
        menuProject.add(item);

        // >>> Plugins <<<
        menuPlugins = PluginManager.getJMenu();
        menuPlugins.setMnemonic(KeyEvent.VK_P);

        add(menuFile);
        add(menuEdit);
        add(menuSearch);
        add(menuProject);
        add(menuPlugins);
    }

    public JMenu getMenuOpen() {
        return menuOpen;
    }

    public void updatePluginsMenu() {
        remove(getMenuCount() - 1);
        menuPlugins = PluginManager.getJMenu();
        add(menuPlugins);
        revalidate();
    }

}



