package application;

import java.util.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.*;

import java.awt.*;
import java.awt.event.*;
import java.io.FileNotFoundException;

import project.*;
import project.ProjectStructure.Tree;
import static logging.Logging.logger;

public class ProjTree extends JTree implements Observer {

    private PopupMenu filePopup = new PopupMenu();
    private PopupMenu projectPopup = new PopupMenu();

    public ProjTree() {
        super(new DefaultMutableTreeNode("root"));
        setRootVisible(false);
        this.addMouseListener(new MouseAction());

        JMenuItem item;
        JMenu menu = new JMenu("New...");
        item = new JMenuItem("File");
        item.addActionListener(new FileAction());
        menu.add(item);
        item = new JMenuItem("Package");
        item.addActionListener(new FileAction());
        menu.add(item);
        filePopup.add(menu);
        item = new JMenuItem("Add File");
        item.addActionListener(new FileAction());
        filePopup.add(item);
        item = new JMenuItem("Open");
        item.addActionListener(new FileAction());
        filePopup.add(item);
        item = new JMenuItem("Delete");
        item.addActionListener(new FileAction());
        filePopup.add(item);
        item = new JMenuItem("Compile");
        item.addActionListener(new FileAction());
        filePopup.add(item);

        item = new JMenuItem("Close");
        item.addActionListener(new FolderAction());
        projectPopup.add(item);
        item = new JMenuItem("Compile");
        item.addActionListener(new FolderAction());
        projectPopup.add(item);
        item = new JMenuItem("Run");
        item.addActionListener(new FolderAction());
        projectPopup.add(item);
        item = new JMenuItem("Run With Arguments");
        item.addActionListener(new FolderAction());
        projectPopup.add(item);
    }

    public void update(Observable obs, Object arg) {

        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                LinkedList <ProjectStructure> list = ProjectManager.getProjectTrees();

                DefaultMutableTreeNode root = (DefaultMutableTreeNode) getModel().getRoot();
                root.removeAllChildren();

                for (ProjectStructure proj : list) {
                    DefaultMutableTreeNode projectNode = new DefaultMutableTreeNode(proj.getTitle());
                    root.add(projectNode);
                    createNodes(projectNode, proj.getTree());
                }

                ((DefaultTreeModel) getModel()).reload(root);
                for (int i = 0; i < getRowCount(); i++)
                    expandRow(i);
                setRootVisible(false);
                repaint();
            }
        });
    }

    private void createNodes(DefaultMutableTreeNode parent, Tree tree) {

        LinkedList<Tree> children = tree.getChildren();

        for (Tree child : children) {
            DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child.getName());
            parent.add(childNode);
            if (child.getType() == Tree.TYPE_FOLDER)
                createNodes(childNode, child);
        }
    }

    private void run(String path, String args) {

        if (! ProjectManager.compileAndRunProject(path, args)) {

            JFileChooser chooser = new JFileChooser();
            FileFilter filter = new FileFilter() {
                public boolean accept(java.io.File f) {
                    return f.getName().toLowerCase().endsWith(".class") || f.isDirectory();
                }
                public String getDescription() {
                    return "Class Files";
                }
            };
            chooser.setFileFilter(filter);
            chooser.setCurrentDirectory(new java.io.File(ProjectManager.getProjectBinPath(path)));

            if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                ProjectManager.runProjectMain(chooser.getSelectedFile().getAbsolutePath(), args);
            } else {
                return;
            }
        }

    }

    private String getProjectPath(TreePath treePath) {
        Object[] nodes = treePath.getPath();

        String path = ((DefaultMutableTreeNode) nodes[1]).toString();

        DefaultMutableTreeNode node = null;
        for (int i = 2; i < nodes.length; i++) {
            node = (DefaultMutableTreeNode) nodes[i];
            path += "/" + node.toString();
        }

        return path;
    }

    public class MouseAction implements MouseListener {

        public void mouseClicked(MouseEvent e) {
            Point loc = e.getPoint();
            setSelectionRow(getClosestRowForLocation(loc.x, loc.y));

            if (e.getModifiers() == 16 && e.getClickCount() >= 2) {
                TreePath treePath = getPathForLocation(loc.x, loc.y);
                if (treePath != null) {
                    String path = getProjectPath(treePath);
                    int index = path.indexOf('/');
                    if (index != -1)
                        ProjectManager.openFile(path.substring(index + 1), path.substring(0, index));
                }
            }

            if (e.getModifiers() == 4) {
                TreePath treePath = ProjTree.this.getPathForLocation(loc.x, loc.y);

                if (treePath != null) {
                    String path = getProjectPath(treePath);
                    if (path.indexOf('/') != -1)
                        filePopup.show(ProjTree.this, loc.x, loc.y);
                    else projectPopup.show(ProjTree.this, loc.x, loc.y);
                }
            }

        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mousePressed(MouseEvent e) {
        }

        public void mouseReleased(MouseEvent e) {
        }

    }

    class FolderAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Point loc = projectPopup.getLocation();
            TreePath treePath = getClosestPathForLocation(loc.x, loc.y);
            if (treePath == null) return;

            String path = getProjectPath(treePath);
            int index = path.indexOf('/');

            if (e.getActionCommand().equals("Close")) {
                if (index > 0) ProjectManager.closeProject(path.substring(0, index));
                else ProjectManager.closeProject(path);
                return;
            }

            if (e.getActionCommand().equals("Compile")) {
                if (index > 0) path = path.substring(0, index);
                ProjectManager.compileProject(path);
                return;
            }

            if (e.getActionCommand().equals("Run")) {
                if (index > 0) run(path.substring(0, index), null);
                else run(path, null);
                return;
            }

            if (e.getActionCommand().equals("Run With Arguments")) {
                if (index > 0) run(path.substring(0, index), JOptionPane.showInputDialog("Arguments:"));
                else run(path, JOptionPane.showInputDialog("Arguments:"));
                return;
            }
        }
    }

    class FileAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Point loc = filePopup.getLocation();
            TreePath treePath = getClosestPathForLocation(loc.x, loc.y);
            if (treePath == null) return;

            String path = getProjectPath(treePath);
            int index = path.indexOf('/');

            if (e.getActionCommand().equals("Open")) {
                String file = path.substring(index + 1),
                       project = path.substring(0, index);
                if (!ProjectManager.isDirectory(file, project))
                    ProjectManager.openFile(file, project);
                return;
            }

            if (e.getActionCommand().equals("Delete")) {
                ProjectManager.removeFileFromProject(path.substring(index + 1), path.substring(0, index));
                return;
            }

            if (e.getActionCommand().equals("Add File")) {
                JFileChooser choose = new JFileChooser();
                if (choose.showOpenDialog(ProjTree.this) == JFileChooser.APPROVE_OPTION)
                    try {
                        java.io.File file = choose.getSelectedFile();
                        String folder = path.substring(index + 1),
                               project = path.substring(0, index);
                        if (!ProjectManager.isDirectory(folder, project))
                            folder = folder.substring(0, folder.lastIndexOf('/'));
                        ProjectManager.addFileToProject(file.getAbsolutePath(), folder + '/' + file.getName(), project);
                    } catch (FileNotFoundException ex) {
                        logger.throwing("KTree", "actionPerformed", ex);
                    }
                return;
            }

            if (e.getActionCommand().equals("File")) {
                String fileName = JOptionPane.showInputDialog("Insert file name"),
                       folder = path.substring(index + 1),
                       project = path.substring(0, index);
                if (fileName == null || fileName.length() == 0) return;
                if (!ProjectManager.isDirectory(folder, project))
                    folder = folder.substring(0, folder.lastIndexOf('/'));
                ProjectManager.addFileToProject(folder + '/' + fileName, project);
                return;
            }

            if (e.getActionCommand().equals("Package")) {
                String packageName = JOptionPane.showInputDialog("Insert package name"),
                       folder = path.substring(index + 1),
                       project = path.substring(0, index);
                if (packageName == null || packageName.length() == 0) return;
                if (!ProjectManager.isDirectory(folder, project))
                    folder = folder.substring(0, folder.lastIndexOf('/'));
                ProjectManager.addFolderToProject(folder + '/' + packageName, project);
                return;
            }

        }

    }

}
