package language.java;

import java.io.*;
import java.util.*;
import javax.tools.*;
import javax.swing.JEditorPane;

import application.Application;
import application.Console;

import project.*;
import static logging.Logging.logger;

public class JavaCompiler {

    public static boolean compile(Iterable<String> files, Project project) {
        javax.tools.JavaCompiler javac = ToolProvider.getSystemJavaCompiler();
        StandardJavaFileManager fileManager = javac.getStandardFileManager(null, null, null);

        LinkedList<String> options = new LinkedList<String>();
        options.add("-d");
        options.add(project.getPathToDir() + "bin");
        options.add("-sourcepath");
        options.add(project.getPathToDir() + "src");
        options.add("-classpath");
        options.add(project.getPathToDir() + "bin");

        Iterable<? extends JavaFileObject> compilationUnits = fileManager.getJavaFileObjectsFromStrings(files);
        DiagnosticCollector<JavaFileObject> diagnosticCollector = new DiagnosticCollector<JavaFileObject>();
        StringWriter out = new StringWriter();

        boolean succes = javac.getTask(out, fileManager, diagnosticCollector, options, null, compilationUnits).call();

        Console console = Application.getConsole();
        JEditorPane errorConsole = console.getErrorConsole();

        String messages = out.toString().trim();
        if (messages.length() > 0) messages = messages + "\n";
        for (Diagnostic<? extends JavaFileObject> d : diagnosticCollector.getDiagnostics()) {
            String message = d.getMessage(null);
            messages = messages + message + "\n";
        }

        if (succes)
            messages = messages + "Compile successful.";

        errorConsole.setText(messages);
        console.setSelectedIndex(0);

        try {
            fileManager.close();
        } catch (IOException e) {
            logger.throwing("Compiler", "compile", e);
        }

        return succes;
    }

    public static boolean isValid(String file) {
        int length = file.length();
        return file.substring(length - 5, length).equals(".java");
    }
}
