package language.java;

import java.awt.Color;
import java.awt.Font;

public class JavaDefaultColorScheme implements JavaColorScheme {

    private static final Color colors[] = new Color[151];
    private static final int styles[] = new int[151];

    public Color getColor(int token) {
        if (token > colors.length || token < 0) return Color.black;
        if (colors[token] == null) return Color.black;
        return colors[token];
    }

    public int getStyle(int token) {
        if (token > styles.length || token < 0) return 0;
        return styles[token];
    }

    {
        colors[COMMENT] = Color.blue;

        colors[TRUE] = colors[FALSE] = colors[THIS] = colors[SUPER] = colors[NULL] = new Color(140, 0, 60);
        styles[TRUE] = styles[FALSE] = styles[THIS] = styles[SUPER] = styles[NULL] = Font.BOLD;

        for (int operator = COMMA; operator <= ARRAY; operator++)
            colors[operator] = new Color(0, 150, 0);

        int[] specialOperators = {INSTANCEOF, NEWARRAY, NEWINSTANCE, NEWFROMNAME, FIELD, NEW, INC, DEC, GOTO};
        for (int op : specialOperators) {
            colors[op] = new Color(140, 0, 60);
            styles[op] = Font.BOLD;
        }

        for (int type = BYTE; type <= BOOLEAN; type++) {
            colors[type] = new Color(0, 160, 50);
            styles[type] = Font.BOLD;
        }

        for (int statement = IF; statement <= VARDECLARATION; statement++) {
            colors[statement] = new Color(140, 0, 60);
            styles[statement] = Font.BOLD;
        }

        for (int value = BOOLEANVAL; value <= STRINGVAL; value++)
            colors[value] = Color.magenta;

        for (int declaration = CLASS; declaration <= INTERFACE; declaration++) {
            colors[declaration] = new Color(0, 160, 50);
            styles[declaration] = Font.BOLD;
        }
        colors[IMPORT] = colors[PACKAGE] = new Color(200, 0, 0);
        styles[IMPORT] = styles[PACKAGE] = Font.BOLD;
        colors[THROWS] = new Color(0, 160, 50);
        styles[THROWS] = Font.BOLD;

        for (int modifier = PRIVATE; modifier <= STRICTFP; modifier++) {
            colors[modifier] = new Color(0, 160, 50);
            styles[modifier] = Font.BOLD;
        }

    }

}
