package language.java;

import java.io.*;
import java.io.File;

import project.*;
import application.Application;
import application.Console;
import static logging.Logging.logger;

public class JavaRunner {
    private static boolean available = true;
    private static Process process = null;

    public static void run(Project p, String mainClass, String userArg) {
        if (!available) return;
        available = false;
        Application.getToolBar().deactivateRun();

        String[] args;
        if (userArg != null && userArg.length() > 0) {
            String[] userArgs = userArg.split("\\s");
            args = new String[userArgs.length + 4];
            args[0] = "java";
            args[1] = "-classpath";
            args[2] = p.getPathToDir() + "bin";
            args[3] = mainClass;
            for (int i = 0; i < userArgs.length; i++)
                args[4 + i] = userArgs[i];
        } else {
            args = new String[4];
            args[0] = "java";
            args[1] = "-classpath";
            args[2] = p.getPathToDir() + "bin";
            args[3] = mainClass;
        }

        ProcessBuilder pb = new ProcessBuilder(args);
        pb.directory(new File(p.getPathToDir()));
        pb.redirectErrorStream(true);

        Console console = Application.getConsole();
        console.setSelectedIndex(1);
        console.getRunnerConsole().grabFocus();

        try {
            process = pb.start();
            console.processStarted(process.getInputStream(), process.getOutputStream());
        } catch (IOException e) {
            logger.throwing("JavaRunner", "run", e);
            return;
        }
    }

    public static void kill() {
        if (process != null) process.destroy();
        ended();
    }

    public static void ended() {
        available = true;
        Application.getToolBar().activateRun();
    }

}
