package language.xml;

import static logging.Logging.logger;

import javax.swing.text.*;

import application.Config;
import language.*;

public class XMLDocument extends XDocument {

    String filePath = null;

    public XMLDocument(String path) {
        super(new GapContent(1024));
        filePath = path;
        setDocumentFilter(new XMLDocumentFilter());
    }

    protected void insertUpdate(DefaultDocumentEvent chng, AttributeSet attr) {
        super.insertUpdate(chng, attr);
    }

    protected void removeUpdate(DefaultDocumentEvent chng) {
        super.removeUpdate(chng);
    }

    void updateComments(Element[] elems) {
        Element root = getDefaultRootElement();
        boolean commentStarted = false;

        int index = root.getElementIndex(elems[0].getStartOffset());
        if (index > 0) {
            AttributeSet attributes = root.getElement(index - 1).getAttributes();
            if (attributes.isDefined(CommentAttribute.comment)) {
                CommentAttribute comm = (CommentAttribute) attributes.getAttribute(CommentAttribute.comment);
                if ((comm.inComment && comm.leftTo < 0) || comm.rightFrom >= 0)
                    commentStarted = true;
            }
        }

        for (Element elem : elems) {
            CommentAttribute comm = null;
            String line = null;

            try {
                line = getText(elem.getStartOffset(), elem.getEndOffset() - elem.getStartOffset());
            } catch (BadLocationException e) {
                // this should never happen
                logger.throwing("XMLDocument", "updateComments", e);
                continue;
            }

            if (commentStarted)
                comm = new CommentAttribute(-1, -1, commentStarted);

            if ((index = line.indexOf("-->")) != -1) {
                if (comm == null)
                    comm = new CommentAttribute(index + 2, -1, commentStarted);
                else comm.setLeftTo(index + 2);
                commentStarted = false;
                line = line.substring(index + 3);
            }

            while (line.length() > 0) {
                if ((index = line.indexOf("<!--")) == -1) break;

                line = line.substring(index + 2);
                int cindex = line.indexOf("-->");
                if (cindex == -1) {
                    if (comm == null)
                        comm = new CommentAttribute(-1, index, commentStarted);
                    else comm.setRightFrom(index);
                    commentStarted = true;
                } else
                    line = line.substring(cindex + 3);
            }

            MutableAttributeSet attributes = (MutableAttributeSet) elem.getAttributes();
            attributes.removeAttribute(CommentAttribute.comment);
            if (comm != null)
                attributes.addAttribute(CommentAttribute.comment, comm);
        }

        index = root.getElementIndex(elems[elems.length - 1].getStartOffset());

        while (true) {
            index ++;
            Element elem = root.getElement(index);
            if (elem == null) break;

            MutableAttributeSet attributes = (MutableAttributeSet) elem.getAttributes();
            if (attributes.isDefined(CommentAttribute.comment)) {
                CommentAttribute comm = (CommentAttribute) attributes.getAttribute(CommentAttribute.comment);
                if (commentStarted) {
                    if (comm.inComment || comm.leftTo >= 0) break;
                    comm.inComment = true;
                } else {
                    if (!comm.inComment) break;
                    comm.inComment = false;
                }
            } else {
                if (!commentStarted) break;
                attributes.addAttribute(CommentAttribute.comment, new CommentAttribute(-1, -1, true));
            }
        }
    }

    static class CommentAttribute {
        private int leftTo;
        private int rightFrom;
        private boolean inComment;

        public static final String comment = "comment";

        public CommentAttribute(int leftTo, int rightFrom, boolean inComment) {
            this.leftTo = leftTo;
            this.rightFrom = rightFrom;
            this.inComment = inComment;
        }

        public void setLeftTo(int leftTo) {
            this.leftTo = leftTo;
        }

        public void setRightFrom(int rightFrom) {
            this.rightFrom = rightFrom;
        }

        public void setinComment(boolean inComment) {
            this.inComment = inComment;
        }

        public boolean isComment(int pos) {
            return (inComment && (pos <= leftTo || leftTo < 0)) || (rightFrom >= 0 && pos >= rightFrom);
        }
    }

    class XMLDocumentFilter extends DocumentFilter {

        private String filter(DocumentFilter.FilterBypass fb,
                int offset, String string) throws BadLocationException {

            PlainDocument doc = (PlainDocument) fb.getDocument();

            if (string.equals("\n") || string.equals("\r\n")) {

                int start = doc.getParagraphElement(offset).getStartOffset();
                String line = doc.getText(start, offset - start);

                StringBuffer buffer = new StringBuffer();
                for (int i = 0; i < line.length(); ++i)
                    if (line.charAt(i) == ' ')
                        buffer.append(' ');
                    else
                        break;

                string += buffer.toString();

            } else if (string.equals("\t")) {

                int start = doc.getParagraphElement(offset).getStartOffset();
                int cnt = (offset - start) % Config.TAB_SIZE;
                cnt = Config.TAB_SIZE - cnt;

                StringBuffer buffer = new StringBuffer();
                for (int i = 0; i < cnt; ++i)
                    buffer.append(' ');

                string = buffer.toString();

            }

            return string;

        }

        public void insertString(DocumentFilter.FilterBypass fb,
                int offset, String string, AttributeSet attr)
                throws BadLocationException {

            string = filter(fb, offset, string);
            super.insertString(fb, offset, string, attr);

        }

        public void replace(DocumentFilter.FilterBypass fb,
                int offset, int length, String text,
                AttributeSet attrs) throws BadLocationException {

            text = filter(fb, offset, text);
            super.replace(fb, offset, length, text, attrs);

        }

    }

}
