package logging;

import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.util.logging.Level;
import java.util.logging.FileHandler;
import java.util.logging.SimpleFormatter;

public class Logging {

    // Everything will use this static field to log.
    public static java.util.logging.Logger logger =
        java.util.logging.Logger.getLogger("global");

    private static FileHandler fileHandler = null;
    private static WindowHandler windowHandler = null;
    private static PrintStream systemOut = null, systemErr = null;
    private static ByteArrayOutputStream outStream = null, errStream = null;
    private static PrintStream customOut = null, customErr = null;
    private static String sourceClassOut = null, sourceMethodOut = null;
    private static String sourceClassErr = null, sourceMethodErr = null;

    // Start logging to a file.
    // Method called only by Main.
    public static void init() {
        systemOut = System.out;
        systemErr = System.err;
        outStream = new ByteArrayOutputStream();
        errStream = new ByteArrayOutputStream();
        customOut = new PrintStream(outStream);
        customErr = new PrintStream(errStream);

        try {
            fileHandler = new FileHandler("messages.log", true);
            fileHandler.setFormatter(new SimpleFormatter());
            logger.addHandler(fileHandler);
        } catch (IOException e) {
            System.err.println("I cannot start the logger.");
        }

        setLevel(Level.ALL);
    }

    // Start logging to the logging window too.
    // Method called only by Main, but not in the final version.
    public static void showLoggingWindow() {
        windowHandler = new WindowHandler();
        windowHandler.setFormatter(new SimpleFormatter());
        logger.addHandler(windowHandler);

        JFrame frame = new JFrame("Logging window");
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

        Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
        dimension.width >>= 1;
        frame.setSize(dimension);

        JScrollPane scroller = new JScrollPane(windowHandler.getTextArea());
        frame.add(scroller);

        frame.setVisible(true);
    }

    // Set the logging level. For development phase, the level will be
    // Level.ALL. For the final version, it will be Level.WARNING.
    // Method called only by Main in the final version, with Level.WARNING.
    public static void setLevel(Level newLevel) {
        logger.setLevel(newLevel);
    }

    // Start redirecting System.out to the logger.
    public static void startLoggingSystemOut(String srcClass, String srcMethod) {
        sourceClassOut = srcClass;
        sourceMethodOut = srcMethod;
        outStream.reset();
        System.setOut(customOut);
    }

    // Stop redirecting System.out to the logger.
    public static void stopLoggingSystemOut() {
        customOut.flush();
        if (outStream.toString().length() > 0)
            logger.logp(Level.FINEST, sourceClassOut, sourceMethodOut,
                    "System.out:\n" + outStream.toString());
        System.setOut(systemOut);
    }

    // Start redirecting System.err to the logger.
    public static void startLoggingSystemErr(String srcClass, String srcMethod) {
        sourceClassErr = srcClass;
        sourceMethodErr = srcMethod;
        errStream.reset();
        System.setErr(customErr);
    }

    // Stop redirecting System.err to the logger.
    public static void stopLoggingSystemErr() {
        customErr.flush();
        if (errStream.toString().length() > 0)
            logger.logp(Level.FINEST, sourceClassErr, sourceMethodErr,
                    "System.err:\n" + errStream.toString());
        System.setErr(systemErr);
    }

}
