package plugins;

import java.awt.Color;
import java.awt.Font;
import java.io.*;

import javax.swing.JFileChooser;
import javax.swing.text.*;
import org.apache.commons.lang.StringEscapeUtils;

import application.*;
import language.java.*;
import language.java.JavaDocument.CommentAttribute;
import static logging.Logging.logger;

public class HTMLOutput extends Plugin {

    public HTMLOutput() {
        super("HTML Output");
        registerAction(new HTMLGenerator());
    }

    public void init() {
    }

    public void clear() {
    }

    public class HTMLGenerator implements ExternAction {

        public String getMessage() {
            return "Generate Html";
        }

        public void run() {
            EditorContent editorContent = (EditorContent) Application.getEditor().getSelectedComponent();
            if (editorContent == null) return;

            EditorPane editorPane = editorContent.getEditorPane();
            String file = editorPane.getPath();
            if (!file.substring(file.length() - 5, file.length()).equals(".java")) return;

            JFileChooser choose = new JFileChooser();
            if (!(choose.showSaveDialog(Application.getFrame()) == JFileChooser.APPROVE_OPTION)) return;

            FileWriter out = null;
            try {
                out = new FileWriter(choose.getSelectedFile());
            } catch (IOException e) {
                return;
            }

            JavaDocument doc = (JavaDocument) editorPane.getDocument();
            JavaColorScheme colors = ((JavaEditorKit) editorPane.getEditorKit()).getStylePreferences().getJavaColorScheme();

            startHTML(out, editorPane.getPath());

            Color lastColor = null;
            int lastStyle = 0,
                lastPos = 0,
                startPos = 0,
                endPos = doc.getLength();

            JavaDocument.Scanner scanner = new JavaDocument.Scanner();

            while (startPos < endPos) {
                scanner.restartScan(doc, startPos);
                Color actColor = colors.getColor(scanner.token);
                int actStyle = colors.getStyle(scanner.token);

                // see if we are in a multi-line comment
                Element elem = doc.getDefaultRootElement();
                elem = elem.getElement(elem.getElementIndex((int) scanner.pos));
                if (elem != null) {
                    AttributeSet attributes = elem.getAttributes();
                    if (attributes.isDefined(CommentAttribute.comment) &&
                            ((CommentAttribute) attributes.getAttribute(CommentAttribute.comment)).
                                                isComment((int) scanner.pos - elem.getStartOffset())) {
                        actColor = colors.getColor(JavaDefaultColorScheme.COMMENT);
                        actStyle = colors.getStyle(JavaDefaultColorScheme.COMMENT);
                    }
                }

                // see if it's a single-line /*...*/ comment
                int start = (int) scanner.pos;
                String token = null;
                try {
                    token = doc.getText(start, (int) scanner.getEndPos() - start);
                } catch (BadLocationException e) {
                    logger.throwing("HTMLOutput", "run", e);
                }
                if (token.indexOf("/*") != -1) {
                    actColor = colors.getColor(JavaDefaultColorScheme.COMMENT);
                    actStyle = colors.getStyle(JavaDefaultColorScheme.COMMENT);
                }

                if (lastColor != null && (actColor != lastColor || actStyle != lastStyle)) {
                    try {
                        insertText(out, doc.getText(lastPos, startPos - lastPos), lastColor, lastStyle);
                    } catch (BadLocationException e) {
                        logger.throwing("HTMLOutput", "run", e);
                    }
                    lastPos = startPos;
                }

                lastColor = actColor;
                lastStyle = actStyle;
                int aux = Math.min((int) scanner.getEndPos(), endPos);
                if (startPos >= aux) startPos = endPos;
                else startPos = aux;

                try {
                    scanner.scan();
                } catch (IOException e) {
                    logger.throwing("HTMLOutput", "run", e);
                }
            }

            try {
                insertText(out, doc.getText(lastPos, endPos - lastPos), lastColor, lastStyle);
            } catch (BadLocationException e) {
                logger.throwing("HTMLOutput", "run", e);
            }

            endHTML(out);
            try {
                out.close();
            } catch (IOException e) {
                logger.throwing("HTMLOutput", "run", e);
            }
        }

        private void insertText(FileWriter out, String text, Color color, int fontStyle) {
            try {
                out.write("<span style=\"color : rgb(" + color.getRed() + ", " + color.getGreen()
                                                        + ", " + color.getBlue() + ")");
                if (fontStyle == Font.BOLD) out.write("; font-weight : bold");
                out.write("\">");
                out.write(StringEscapeUtils.escapeHtml(text));
                out.write("</span>");
            } catch (IOException e) {
                logger.throwing("HTMLOutput", "insertText", e);
            }
        }

        private void startHTML(FileWriter out, String name) {
            try {
                out.write("<html>\n");
                out.write("    <head>\n");
                out.write("        <title>" + name + "</title>\n");
                out.write("    </head>\n");
                out.write("    <body>\n");
                out.write("        <pre>\n");
            } catch (IOException e) {
                logger.throwing("HTMLOutput", "startHTML", e);
            }
        }

        private void endHTML(FileWriter out) {
            try {
                out.write("        </pre>\n");
                out.write("    </body>\n");
                out.write("</html>\n");
            } catch (IOException e) {
                logger.throwing("HTMLOutput", "startHTML", e);
            }
        }
    }
}
