package plugins;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.JOptionPane;
import application.*;

public class SEOChecking extends Plugin {

    public SEOChecking() {
        super("SEO Checking");
        registerAction(new SEOGenerator());
    }

    public void init() {
    }

    public void clear() {
    }
    
    public class SEOGenerator implements ExternAction {

		public String getMessage() {
			return "SEO Checking";
		}

		public void run() {
			EditorContent editorContent = (EditorContent) Application.getEditor().getSelectedComponent();
            if (editorContent == null) return;

            EditorPane editorPane = editorContent.getEditorPane();
            String file = editorPane.getPath();
            if (!file.substring(file.length() - 5, file.length()).equals(".html")) return;
			
            //data contains the html text
            String data = editorPane.getText();
            
            //messages
            String titleMessage = "";
            String metaKeywordsMessage = "";
            String metaDescriptionMessage = "";
            String scriptMessage = "";
            String h1Message = "";
            
            //regexp
            Pattern MY_TITLE_PATTERN = Pattern.compile("<title>([^<]*)</title>");
            Pattern MY_METAKEY_PATTERN = Pattern.compile("<meta name=\"Keywords\" content=\"([^<]*)\"([^<]*)>");
            Pattern MY_METADESC_PATTERN = Pattern.compile("<meta name=\"Description\" content=\"([^<]*)\"([^<]*)>");
            Pattern MY_SCRIPT_PATTERN = Pattern.compile("<head>(?:<(?!script)|[^<]*)*<script(?:<(?!/head)|[^<]*)*</head>");
            //Pattern MY_H1_PATTERN = Pattern.compile("");
            
            //checks for title tag
            Matcher title = MY_TITLE_PATTERN.matcher(data);
            if (title.find()) {
            	
                data = title.group(1);                 
                //check title length
                if(data.length() < 1)
                	titleMessage = "Insert a title on your page";
                else {
                	if(data.length() > 14) 
                		titleMessage = "Google will consider your title too long";
                	else {
                		titleMessage = "Your title length is OK";
                	}
               }	
            }
            else {
            	titleMessage = "Your page doesn't have title";
            }
            
            //reset data
            data = editorPane.getText();
            
            //check if script tag is in head tag
            Matcher script = MY_SCRIPT_PATTERN.matcher(data);
            if(script.find()) {
            	scriptMessage = "Your script tag has been found in head tag";
            } else {
            	scriptMessage = "For good practices insert your script tag in head tag";
            }
            
            //reset data
            data = editorPane.getText();
            
            //check if meta description tag exists
            //check content length
            Matcher metaDesc = MY_METADESC_PATTERN.matcher(data);
            if(metaDesc.find()) {
            	data = metaDesc.group(1);
            	//check description length
                if(data.length() < 1)
                	metaDescriptionMessage = "Insert a description in content";
                else {
                	if(data.length() > 100) 
                		metaDescriptionMessage = "Google will consider your description too long";
                	else {
                		metaDescriptionMessage = "Your description length is OK";
                	}
               }	
            } else {
            		metaDescriptionMessage = "Your page doesn't have a meta description tag";
            }
            
            //reset data
            data = editorPane.getText();
            
            //check if meta keywords tag exists
            //warning about content if the content length < 1
            Matcher metaKey = MY_METAKEY_PATTERN.matcher(data);
            if(metaKey.find()) {
            	data = metaKey.group(1);
            	//check content length
            	if(data.length() < 1) 
            		metaKeywordsMessage = "Insert some keywords in content";
            	else {
            		metaKeywordsMessage = "Your page have a meta keywords tag";
            	}
            } else {
            		metaKeywordsMessage = "Your page doesn't have a meta keywords tag";
            }
            
            //reset data
            data = editorPane.getText();
            
            //check if h1 is the only one in body content
            //TODO
            
            JOptionPane.showMessageDialog(Application.getFrame(), titleMessage + "\n" +
            							  metaKeywordsMessage + "\n" +
            							  metaDescriptionMessage + "\n" +
            							  scriptMessage + "\n" +
            							  h1Message, "BASIC SEO Checker", 1 ); 
		}	
    }   
}
