package project;

import java.util.*;
import java.io.IOException;
import static logging.Logging.logger;

public class File extends Observable {

    private String path = null;
    private String content = null;
    private String contentType = null;

    public File(String path) throws IOException {

        this.path = path;

        java.io.File f = new java.io.File(path);

        if (f.isDirectory())
            throw new java.io.IOException("Cannot open. The file " + path + " exists and it's a directory");

        if (!f.exists())
            f.createNewFile();

        contentType = "text/plain";

        int index = path.lastIndexOf('.');
        if (index != -1) {
            String extension = path.substring(index + 1);
            if (extension.equals("java"))
                contentType = "text/java";
            else if (extension.equals("xml"))
                contentType = "text/xml";
            else if (extension.equals("html"))
            	contentType = "text/html";
        }
    }

    public void copy(String source) {

        if (source == null) return;

        java.io.File inFile = new java.io.File(source);
        java.io.FileReader in;
        java.io.FileWriter out;

        try {
            in = new java.io.FileReader(inFile);
            out = new java.io.FileWriter(path);
        } catch (java.io.IOException e) {
            logger.throwing("project.File", "save", e);
            return;
        }

        char[] c = new char[(int)inFile.length()];
        try {
            in.read(c);
            out.write(c);
            in.close();
            out.close();
        } catch (java.io.IOException e) {
            logger.throwing("project.File", "load", e);
            return;
        }

    }

    public static boolean exists(String path) {
        java.io.File f = new java.io.File(path);
        return f.exists();
    }

    public static void renameFile(String from, String to) {
        (new java.io.File(from)).renameTo(new java.io.File(to));
    }

    public static void recreate(String path) {
        java.io.File file = new java.io.File(path);
        delete(file);
        file.mkdir();
    }

    public static void delete(String path) {
        delete(new java.io.File(path));
    }

    private static void delete(java.io.File file) {
        java.io.File[] children = file.listFiles();
        if (children != null)
            for (java.io.File child : children)
                delete(child);
        file.delete();
    }

    public void load() {

        java.io.File file;
        java.io.FileReader in;

        try {
            file = new java.io.File(path);
            in = new java.io.FileReader(file);
        } catch (java.io.FileNotFoundException e) {
            logger.throwing("project.File", "load", e);
            return;
        }

        char[] c = new char[(int)file.length()];
        try {
            in.read(c);
            in.close();
        } catch (java.io.IOException e) {
            logger.throwing("project.File", "load", e);
            return;
        }

        content = new String(c);

        setChanged();
        notifyObservers();
    }

    public void save(String data) {

        try {
            java.io.FileWriter out = new java.io.FileWriter(path);
            char[] c = data.toCharArray();
            out.write(c);
            out.close();
        } catch (java.io.IOException e) {
            logger.throwing("project.File", "save", e);
            return;
        }
    }

    public String getPath() {
        return path;
    }

    public String getName() {
        if (path == null)
            return null;
        return path.substring(path.lastIndexOf('/') + 1);
    }

    public String getContent() {
        return content;
    }

    public String getContentType() {
        return contentType;
    }

}
