package project;

import java.util.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;

import com.sun.tools.javac.tree.JCTree;
import com.sun.tools.javac.util.Context;
import com.sun.tools.javac.util.Options;
import com.sun.tools.javac.code.Flags;
import static com.sun.tools.javac.tree.JCTree.*;

import java.io.IOException;
import language.java.JavaCompiler;

import logging.Logging;
import static logging.Logging.logger;

public class Project extends Observable {

    private String path = null;
    private Document doc = null;

    public Project(String path, String newTitle) throws IOException {

        this.path = path;
        if (newTitle == null)
            newTitle = "Null";

        int index = path.lastIndexOf('.');
        if (index <= 0 || !path.substring(index + 1).equals("proj"))
            throw new IOException("Wrong type of file!");

        index = path.lastIndexOf('/');
        if (0 < index)
            (new java.io.File(path.substring(0, index))).mkdir();

        java.io.File f = new java.io.File(path);

        if (!f.exists()) {

            f.createNewFile();

            // Create a minimal XML file as project file.
            // Tags <project>, <title> and <tree> are binding.
            Element root, title, config, tree;

            try {
                doc = DocumentBuilderFactory.newInstance().
                    newDocumentBuilder().newDocument();
            } catch (ParserConfigurationException e) {
                logger.throwing("project.Project", "Project", e);
            }

            root = doc.createElement("project");
            root.appendChild(doc.createTextNode("\n"));

            title = doc.createElement("title");
            title.appendChild(doc.createTextNode(newTitle.toString()));
            root.appendChild(title);
            root.appendChild(doc.createTextNode("\n"));

            config = doc.createElement("config");
            root.appendChild(config);
            root.appendChild(doc.createTextNode("\n"));

            tree = doc.createElement("tree");
            tree.appendChild(doc.createTextNode("\n"));
            root.appendChild(tree);
            root.appendChild(doc.createTextNode("\n"));

            doc.appendChild(root);

            saveToDisk();

        }
    }

    public void load() {
        try {
            doc = DocumentBuilderFactory.newInstance().
                newDocumentBuilder().parse(new java.io.File(path));

            Node root = doc.getElementsByTagName("tree").item(0);
            NodeList children = root.getChildNodes();
            boolean hasSrcFolder = false;
            for (int i = 0; i < children.getLength(); i++) {
                Node child =  children.item(i), name;
                if (child.getNodeName().equals("folder") && child.hasAttributes()
                        && (name = child.getAttributes().getNamedItem("name")) != null
                        && name.getNodeValue().equals("src"))
                    hasSrcFolder = true;
            }

            if (!hasSrcFolder)
                ProjectManager.addFolderToProject("src", getTitle());

            setChanged();
            notifyObservers();
        } catch (Exception e) {
            logger.throwing("project.Project", "load", e);
            return;
        }
    }

    public void addFile(String file) {

        Node root = getParent(file);
        if (root == null)
            logger.severe("The file " + file + "cannot be added: parent doesn't exist in the project.");

        file = file.substring(file.lastIndexOf('/') + 1);

        // see if the file already exists there
        NodeList children = root.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i), name;
            if (child.hasAttributes() && (name = child.getAttributes().getNamedItem("name")) != null
                    && name.getNodeValue().equals(file)) {
                logger.info("The file " + file + " already exists in the project.");
                return;
            }
        }

        // add the file
        Element fileNode = doc.createElement("file");
        fileNode.setAttribute("name", file);
        root.appendChild(fileNode);
        root.appendChild(doc.createTextNode("\n"));

        saveToDisk();
        setChanged();
        notifyObservers();
    }

    public void removeFile(String file) {

        Node root = getParent(file);
        NodeList children = root.getChildNodes();

        file = file.substring(file.lastIndexOf('/') + 1);
        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i), name;
            if (child.hasAttributes() && (name = child.getAttributes().getNamedItem("name")) != null
                    && name.getNodeValue().equals(file)) {
                root.removeChild(child);
                while ((child = children.item(++i)) != null && child.getNodeType() == Node.TEXT_NODE)
                    root.removeChild(child);
                break;
            }
        }

        saveToDisk();
        setChanged();
        notifyObservers();
    }

    public void addFolder(String folder) {

        Node root = getParent(folder);
        if (root == null)
            logger.severe("The folder " + folder + "cannot be added: parent doesn't exist in the project.");

        folder = folder.substring(folder.lastIndexOf('/') + 1);

        // see if the folder already exists there
        NodeList children = root.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i), name;
            if (child.hasAttributes() && (name = child.getAttributes().getNamedItem("name")) != null
                    && name.getNodeValue().equals(folder)) {
                logger.info("The folder " + folder + " already exists in the project.");
                return;
            }
        }

        // add the folder
        Element folderNode = doc.createElement("folder");
        folderNode.setAttribute("name", folder);
        folderNode.appendChild(doc.createTextNode("\n"));
        root.appendChild(folderNode);
        root.appendChild(doc.createTextNode("\n"));

        saveToDisk();
        setChanged();
        notifyObservers();
    }

    private void saveToDisk() {
        DOMSource source = new DOMSource(doc);
        StreamResult result = new StreamResult(new java.io.File(path));
        try {
            TransformerFactory.newInstance().newTransformer().
                transform(source, result);
        } catch (TransformerException e) {
            logger.throwing("project.Project", "saveToDisk", e);
            return;
        }
    }

    public String getMainClass() {

        String mainClass = null;

        LinkedList <String> files = getFilePaths();
        for (String path : files)
            if (path.endsWith(".java")) {

                // Redirect System.out and System.err to the logger.
                Logging.startLoggingSystemOut("language.java.JavaParser", "run");
                Logging.startLoggingSystemErr("language.java.JavaParser", "run");

                // Parse the file for a main class.
                com.sun.tools.javac.main.JavaCompiler compiler =
                    new com.sun.tools.javac.main.JavaCompiler(new Context());
                try {
                    JCCompilationUnit tree = compiler.parse(path); // FIXME: deprecated

                    for (JCTree t : tree.defs)
                        if (t.tag == CLASSDEF) {

                            for (JCTree tt : ((JCClassDecl) t).defs)
                                if (tt.tag == METHODDEF) {
                                    JCMethodDecl method = (JCMethodDecl) tt;

                                    if (method.getName() != null && method.getName().toString().equals("main") &&
                                            method.mods != null &&
                                            (method.mods.flags & Flags.PUBLIC) != 0 &&
                                            (method.mods.flags & Flags.STATIC) != 0 &&
                                            method.getParameters() != null &&
                                            method.getParameters().length() == 1 &&
                                            method.getParameters().get(0).getType().toString().equals("String[]") &&
                                            method.getReturnType().toString().equals("void"))
                                        if (mainClass == null) {
                                            mainClass = path.substring((getPathToDir() + "src/").length());

                                            int index = mainClass.lastIndexOf("/");
                                            if (index == -1) mainClass = "";
                                            else mainClass = mainClass.substring(0, index + 1);

                                            mainClass = mainClass + ((JCClassDecl) t).getSimpleName().toString() + ".class";
                                        }
                                        else
                                            return null;
                                }
                        }
                } catch (IOException e) {
                    logger.throwing("project.Project", "getMainClass", e);
                } catch (Exception e) {
                    logger.throwing("project.Project", "getMainClass", e);
                }

                // Restore System.out and System.err.
                Logging.stopLoggingSystemOut();
                Logging.stopLoggingSystemErr();

            }

        return mainClass;

    }

    public String getPath() {
        return path;
    }

    private Node getParent(String path) {
        Node root = doc.getElementsByTagName("tree").item(0);
        int index;

        while ((index = path.indexOf('/')) != -1) {
            NodeList children = root.getChildNodes();
            int nr = children.getLength(), i;

            for (i = 0; i < nr; i++) {
                Node child = children.item(i), name;
                if (child.getNodeName().equals("folder") && child.hasAttributes()
                        && (name = child.getAttributes().getNamedItem("name")) != null
                        && name.getNodeValue().equals(path.substring(0, index))) {
                    root = child;
                    path = path.substring(index + 1);
                    break;
                }
            }

            if (i == nr) {
                return null;
            }
        }
        return root;
    }

    public LinkedList<String> getAllFilePaths() {
        LinkedList<String> list = new LinkedList<String>();
        String dirPath = getPathToDir();
        getAllFilePaths(list, getPathToDir() + "src", doc.getElementsByTagName("tree").item(0), dirPath.substring(0, dirPath.length() - 1));
        return list;
    }

    private void getAllFilePaths(LinkedList<String> list, String file, Node root, String path) {
        NodeList children = root.getChildNodes();

        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i), name;

            if (child.hasAttributes() && (name = child.getAttributes().getNamedItem("name")) != null) {
                String childPath = path + "/" + name.getNodeValue();
                if (file.indexOf(childPath) == 0 || childPath.indexOf(file) == 0)
                    if (child.getNodeName().equals("folder"))
                        getFilePaths(list, file, child, childPath);
                    else
                        if (child.getNodeName().equals("file"))
                            list.add(childPath);
            }

        }
    }

    public LinkedList<String> getFilePaths() {
        return getFilePaths("src");
    }

    public LinkedList<String> getFilePaths(String file) {
        LinkedList<String> list = new LinkedList<String>();
        String dirPath = getPathToDir();
        getFilePaths(list, getPathToDir() + file, doc.getElementsByTagName("tree").item(0), dirPath.substring(0, dirPath.length() - 1));
        return list;
    }

    private void getFilePaths(LinkedList<String> list, String file, Node root, String path) {
        NodeList children = root.getChildNodes();

        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i), name;

            if (child.hasAttributes() && (name = child.getAttributes().getNamedItem("name")) != null) {
                String childPath = path + "/" + name.getNodeValue();
                if (file.indexOf(childPath) == 0 || childPath.indexOf(file) == 0)
                    if (child.getNodeName().equals("folder"))
                        getFilePaths(list, file, child, childPath);
                    else
                        if (child.getNodeName().equals("file")) {
                            if (JavaCompiler.isValid(childPath))
                                list.add(childPath);
                        }
            }

        }
    }

    public String getPathToDir() {
        int index = path.lastIndexOf('/');
        if (index == -1) return "";
        else return path.substring(0, index + 1);
    }

    public String getTitle() {
        return doc.getElementsByTagName("title").item(0).getTextContent();
    }

    public ProjectStructure getStructure() {
        ProjectStructure structure = new ProjectStructure();
        structure.setTitle(doc.getElementsByTagName("title").item(0).getTextContent());
        structure.setPath(path);
        structure.setTree(ProjectStructure.Tree.constructFromDoc(doc));
        return structure;
    }

}
