package project;

import java.util.*;
import org.w3c.dom.*;
import static logging.Logging.logger;

public class ProjectStructure {

    String title = null, path = null;
    Tree tree = null;

    public void setTitle(String title) {
        this.title = title;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public void setTree(Tree tree) {
        this.tree = tree;
    }

    public String getTitle() {
        return title;
    }

    public String getPath() {
        return path;
    }

    public Tree getTree() {
        return tree;
    }

    public static class Tree {

        public static final int TYPE_FILE = 0;
        public static final int TYPE_FOLDER = 1;

        String name = null; int type;
        LinkedList <Tree> children = null;

        public Tree(String name, int type) {
            this.name = name;
            this.type = type;
            children = new LinkedList <Tree> ();
        }

        public void addChild(Tree child) {
            children.add(child);
        }

        public String getName() {
            return name;
        }

        public int getType() {
            return type;
        }

        public LinkedList <Tree> getChildren() {
            return children;
        }

        public static Tree constructFromDoc(Document doc) {
            NodeList list = doc.getElementsByTagName("tree");
            if (list.getLength() <= 0) {
                logger.severe("No <tree> tag in the project file!");
                return null;
            }
            return createTree(list.item(0));
        }

        private static Tree createTree(Node root) {
            String name = "/";
            if (root.getAttributes().getNamedItem("name") != null)
                name = root.getAttributes().getNamedItem("name").getNodeValue();
            Tree tree = new Tree(name, TYPE_FOLDER);

            NodeList list = root.getChildNodes();
            for (int i = 0; i < list.getLength(); ++i)
                if (list.item(i).getNodeName().equals("file")) {
                    tree.addChild(new Tree(list.item(i).getAttributes().
                            getNamedItem("name").getNodeValue(), TYPE_FILE));
                } else if (list.item(i).getNodeName().equals("folder")) {
                    tree.addChild(createTree(list.item(i)));
                }

            return tree;
        }

    }

}
